#ifndef PERLUPB_ALL_DESCRIPTORS_H
#define PERLUPB_ALL_DESCRIPTORS_H

#include "xs/descriptor/enum.h"
#include "xs/descriptor/enum_value.h"
#include "xs/descriptor/field.h"
#include "xs/descriptor/file.h"
#include "xs/descriptor/message.h"
#include "xs/descriptor/method.h"
#include "xs/descriptor/oneof.h"
#include "xs/descriptor/service.h"
#include "xs/descriptor_pool/add.h"
#include "xs/descriptor_pool/find.h"
#include "xs/descriptor_pool/pool.h"
#include "xs/message/access.h"
#include "xs/message/message.h"
#include "xs/message/serialize.h"
#include "xs/protobuf/arena.h"
#include "xs/protobuf/message.h"

#endif  // PERLUPB_ALL_DESCRIPTORS_H
